import matplotlib.pyplot as plt
import numpy as np

from cratermaker import Crater, Morphology

crater = Crater.maker(final_radius=1.0e3)
morphology = Morphology.maker("simplemoon")

rc = crater.final_radius
rvals = np.linspace(0, 3.0 * rc, 1000)
hvals = morphology.crater_profile(crater, rvals)
ejvals = morphology.ejecta_profile(crater, rvals)

# Filter out the interior of the crater and add the ejecta to the crater profile to draw the combined profile
rej = rvals[ejvals > 0]
ejvals = hvals[ejvals > 0] + ejvals[ejvals > 0]

rnorm = rvals / rc
size_ratio = (hvals.max() - hvals.min()) / (2 * rvals.max())

w = 200
h = w * size_ratio

fig, ax = plt.subplots(figsize=(w, h), layout="constrained")


ax.set_aspect("equal")
linewidth = 16
ax.plot(-rnorm[rnorm <= 1], hvals[rnorm <= 1] / rc, color="black", linewidth=linewidth)
ax.plot(-rnorm[rnorm >= 1], ejvals / rc, color="black", linewidth=linewidth)
ax.plot(rnorm[rnorm <= 1], hvals[rnorm <= 1] / rc, color="black", linewidth=linewidth)
ax.plot(rnorm[rnorm >= 1], ejvals / rc, color="black", linewidth=linewidth)
ax.axis("off")
plt.savefig("morphology_profile.svg", transparent=True)



"""
Run a simulation of the Moon and visualize with PyVista
=======================================================

.. rubric:: By Dennise Valadez and David Minton

This example demonstrates how to run a crater population simulation on the Moon and visualize the final surface using PyVista.

We will use the ``run`` method in a Cratermaker Simulation object for an age of 4.31 billion years using the default Neukum production function [#]_. This will produces a cratering history that is similar to that seen on the most ancient terrains on the Moon. However, keep in mind that this is not necessarily the "true" age of the surface, as the Neukum production function is very poorly constrained prior to 3.9 billion years ago. Also, because we are not setting a random seed, the exact crater population will be different each time the simulation is run.

We also reduce the gridlevel to 6 to speed up the simulation for this example. Even at low resolution, it will take several minutes to run the simulation. We will plot them using a reversed grayscale colormap to somewhat mimic the appearence of the lunar surface, or at least what the Moon would look like if every deep crater was filled with mare basalt!



References
----------

.. [#] Neukum, G., Ivanov, B.A., Hartmann, W.K., (2001) Cratering Records in the Inner Solar System in Relation to the Lunar Reference System. Space Science Reviews 96, 55-86. `doi: 10.1023/A:1011989004263 <https://doi.org/10.1023/A:1011989004263>`
"""

import pyvista as pv

import cratermaker as cm

# Initialize a quick Moon simulation. We will reduce the resolution to gridlevel 6 to speed up the simulation for this example. We set `ask_overwrite` to False to avoid being prompted during the example.
sim = cm.Simulation(target="Moon", gridlevel=6, ask_overwrite=False)

sim.run(age=4310)
sim.export(driver="VTK")
# Load and visualize the resulting surface using Pyvista
mesh = pv.read(sim.surface.output_dir / "surface000001.vtp")

plotter = pv.Plotter()
plotter.add_mesh(mesh, scalars="face_elevation", cmap="Greys_r", show_edges=False)
plotter.show()



"""
Export Cratermaker data to GeoPackage and visualize with GeoPandas
==================================================================

.. rubric:: By David Minton

Cratermaker can export Simulation data to a variety of GIS vector formats, including GeoPackage, ESRI Shapefile, and more. We do this using the GeoPandas library, which provides a set of useful tools for working with geospatial data in Python. Using the "driver" argument of the `export` method, you can export to just about any format supported by GeoPandas (see `here <https://geopandas.org/en/stable/docs/reference/api/geopandas.GeoDataFrame.to_file.html>`__). This is the same tool used to export data as a mesh in VTK format using the "VTK" driver, which does not use GeoPandas.

 One of the most versitile and efficient formats is GeoPackage (GPKG), which is a single file that can store multiple layers of vector data. This can be read by most GIS software, including QGIS and ArcGIS.  In this example, we will run a simple Cratermaker simulation, export the data to a GeoPackage file, and visualize it using GeoPandas. One of the disandvantages of the current implementation is that when exporting the global surface, faces that cross the antimeridian are not exported due an issue with the UxArray library used to handle the conversion. This will be fixed in a future release.

We will also show how to extract a local region of interest and plot it. The default CRS of local regions is a Azimuthal Equidistant projection centered on the local location, which is well-suited for small areas. Because this does not have the antimeridian issue, we can export and visualize the entire local region without any problems.


"""

import geopandas as gpd
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import colormaps

import cratermaker

# Run a lunar simulation for 4 Gy and export the global surface to GeoPackage
sim = cratermaker.Simulation(gridlevel=6, ask_overwrite=False)
sim.run(age=4e3)
sim.export(driver="GPKG")

gdf = gpd.read_file(sim.surface.output_dir / "surface000001.gpkg")
gdf.plot("face_elevation", cmap="Greys_r")


# Extract a 1000km radius region at the equator/prime merdian and export to GeoPackage
local = sim.surface.extract_region(location=(0, 0), region_radius=1000e3)
# Pass the interval number explicitly so that the file number matches the last frame of the simulation
local.save(interval_number=sim.interval_number)
local.export(driver="GPKG")
gdf = gpd.read_file(sim.surface.output_dir / "local_surface000001.gpkg")
gdf.plot("face_elevation", cmap="Greys_r")
print("Done")



"""
Manually emplace a single crater on the Moon and visualize it with PyVista
==========================================================================

This example shows how to emplace a single large crater on the Moon using Cratermaker's `Simulation.emplace()` method. The crater is defined with a specific diameter and location. The final surface is exported and visualized using PyVista.

"""

import pyvista as pv

import cratermaker as cm

sim = cm.Simulation(ask_overwrite=False)
sim.emplace(final_diameter=500e3, location=(45, 60))
sim.export(driver="VTK")
mesh = pv.read(sim.surface.output_dir / "surface000000.vtp")
plotter = pv.Plotter()
plotter.add_mesh(mesh, scalars="face_elevation", cmap="cividis", show_edges=False)
plotter.show()



"""
Run a simulation of a local region on Mars.
===========================================

.. rubric:: By David Minton and Dennise Valadez

This example demonstrates how to use the HiResLocal Surface to run a crater population over a small region of a planet, in this case Mars.

For this example, we will The simulation uses the default Mars configuration and runs for 2 billion years on a local region with a radius of 20 km and resolution of 100 m/pix.  We then visualize the surface using PyVista and a Mars-like colormap.


"""

import pyvista as pv

import cratermaker as cm

# Run a simple Mars sim
sim = cm.Simulation(
    target="Mars",
    surface="hireslocal",
    local_location=(0, 0),
    pix=100.0,
    local_radius=20.0e3,
    ask_overwrite=False,
    rng_seed=86186233406,  # This will ensure we get the same crater population each time we run the example
)
sim.run(age=2000)

sim.export(driver="VTK")
mesh = pv.read(sim.surface.output_dir / "local_surface000001.vtp")
plotter = pv.Plotter()
plotter.add_mesh(mesh, scalars="face_elevation", cmap="pink", show_edges=False)
# Orient the camera to look down on the surface
plotter.camera_position = "yz"
plotter.show()
